import logging

import requests
from django.conf import settings

from .exceptions import ServerError
from .utils import django_files_to_requests_files  # Depricated

logger = logging.getLogger(__name__)

SETTINGS = {
    'DEFAULT_TIMEOUT': 10,
}

SETTINGS.update(getattr(settings, 'DJANGO_ON_CHAIN', {}))


def send_request(
        method,
        url,
        data=None,
        json=None,
        params=None,
        files=None,
        headers=None,
        expected_content_type='application/json',
        expected_error_statuses=None,
        timeout=None):

    timeout = timeout or SETTINGS['DEFAULT_TIMEOUT']
    if isinstance(method, str):
        method = getattr(requests, method)

    try:
        response = method(url, headers=headers, data=data, json=json, files=files, params=params, timeout=timeout)
    except requests.Timeout:
        logger.exception('API call timed out', exc_info={
            'url': url,
            'request_body': data or json,
        })
        raise ServerError()

    check_response_for_errors(response, expected_content_type, expected_error_statuses)
    return response


def check_response_for_errors(
        response,
        expected_content_type='application/json',
        expected_error_statuses=None):

    if response.headers.get('content-type') != expected_content_type:
        logger.error('Invalid content-type returned: {}'.format(response.headers.get('content-type')),
                     exc_info={
                         'url': response.url,
                         'request_body': response.request.body,
                     })
        raise ServerError()

    if response.status_code >= 400 and response.status_code not in (expected_error_statuses or []):
        logger.error('Unexpected status code returned: {}'.format(response.status_code),
                     exc_info={
                         'url': response.url,
                         'request_body': response.request.body,
                     })
        raise ServerError()



import logging

logger = logging.getLogger(__name__)


class ErrorMiddleware(object):
    @staticmethod
    def process_exception(request, exception):
        logger.exception('Uncaught exception', extra={
            'request': request,
            'exception': exception,
        })
        return None



from django.utils.translation import get_language

try:
    from urlparse import parse_qs, urlparse
except ImportError:  # For Python 3
    from urllib.parse import parse_qs, urlparse


def django_files_to_requests_files(files):
    return {k: (f.name, f.read(), f.content_type) for k, f in files.items()} if files is not None else None


def append_lang_param(url, param_name='l'):
    query = urlparse(url).query
    if param_name not in parse_qs(query):
        if url[-1:][0] not in ['?', '&']:
            url += '&' if query else '?'
        url += '{}={}'.format(param_name, get_language())
    return url



"""
Django REST framework handles the following exceptions, and automatically generates error responses:
  - Subclasses of APIException raised inside REST framework,
  - Django's Http404 exception,
  - and Django's PermissionDenied exception.

More info: http://www.django-rest-framework.org/api-guide/exceptions/

We capture them here, and reformat responses to our desired style.
"""
from django.core.exceptions import PermissionDenied
from django.http import Http404
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers, status
from rest_framework.exceptions import APIException
from rest_framework.response import Response
from rest_framework.views import exception_handler

from django_on_chain.exceptions import ServiceError


def reformat_error_handler(e, context):
    response = exception_handler(e, context)  # Call default exception handler to get the standard error response

    reformat_errors = getattr(context.get('view', object()), 'reformat_errors', False)

    if reformat_errors:
        # Order is important
        return reformat_service_error(e) or \
            reformat_drf_validation_error(e) or \
            reformat_drf_api_exception(e) or \
            reformat_django_http_404(e) or \
            reformat_django_permission_denied(e) or \
            response

    return response


def reformat_service_error(e):
    if isinstance(e, ServiceError):
        return Response({
            'type': e.type,
            'message': e.message,
        }, status=e.suggested_http_status or status.HTTP_500_INTERNAL_SERVER_ERROR)

    return None


def reformat_drf_validation_error(e):
    if isinstance(e, serializers.ValidationError):
        return Response({
            'type': 'bad_request',
            'message': _('Invalid request: {}').format(e.detail),
        }, status=status.HTTP_400_BAD_REQUEST)

    return None


def reformat_drf_api_exception(e):
    if isinstance(e, APIException):
        return Response({
            'type': e.default_code,
            'message': force_text(e.default_detail),
        }, status=e.status_code)

    return None


def reformat_django_http_404(e):
    if isinstance(e, Http404):
        return Response({
            'type': 'not_found',
            'message': _('Resource was not found'),
        }, status=status.HTTP_404_NOT_FOUND)

    return None


def reformat_django_permission_denied(e):
    if isinstance(e, PermissionDenied):
        return Response({
            'type': 'access_forbidden',
            'message': _('You do not have permission to perform the action requested'),
        }, status=status.HTTP_403_FORBIDDEN)
    return None



from django.utils import translation

try:
    from django.utils.deprecation import MiddlewareMixin as base_class
except ImportError:  # For django 1.8.14
    base_class = object


class LocaleMiddleware(base_class):
    @staticmethod
    def process_request(request):
        lang = request.GET.get('lang', 'en')
        translation.activate(lang)



#!/usr/bin/env python
from codecs import open
from os import path
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'requirements.in')) as f:
    requirements = f.read().split('\n')

setup(
    name='django-on-chain',
    version='0.0.2',
    description='Django On Chain',
    long_description='The D is silent.',
    license='Closed-Source Proprietary License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: Other/Proprietary License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    packages=find_packages(exclude=['app', 'tests']),
    install_requires=requirements,
)
